@@ -4,13 +4,16 @@
 #ifndef QEVENT_H
 #define QEVENT_H
 
+#if 0
+#pragma qt_class(QtEvents)
+#endif
+
 #include <QtGui/qtguiglobal.h>
 
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qiodevice.h>
 #include <QtCore/qlist.h>
 #include <QtCore/qnamespace.h>
-#include <QtCore/qpointer.h>
 #include <QtCore/qstring.h>
 #include <QtCore/qurl.h>
 #include <QtCore/qvariant.h>
@@ -30,8 +33,12 @@ QT_BEGIN_NAMESPACE
 class QFile;
 class QAction;
 class QMouseEvent;
+template <typename T> class QPointer;
 class QPointerEvent;
 class QScreen;
+#if QT_CONFIG(shortcut)
+class QShortcut;
+#endif
 class QTabletEvent;
 class QTouchEvent;
 #if QT_CONFIG(gestures)
@@ -74,7 +81,7 @@ public:
         return pointingDevice() ? pointingDevice()->pointerType() : QPointingDevice::PointerType::Unknown;
     }
     void setTimestamp(quint64 timestamp) override;
-    qsizetype pointCount() const { return m_points.count(); }
+    qsizetype pointCount() const { return m_points.size(); }
     QEventPoint &point(qsizetype i);
     const QList<QEventPoint> &points() const { return m_points; }
     QEventPoint *pointById(int id);
@@ -845,7 +852,10 @@ public:
 
     inline QString file() const { return m_file; }
     QUrl url() const { return m_url; }
+#if QT_DEPRECATED_SINCE(6, 6)
+    QT_DEPRECATED_VERSION_X_6_6("Interpret the string returned by file()")
     bool openFile(QFile &file, QIODevice::OpenMode flags) const;
+#endif
 private:
     QString m_file;
     QUrl m_url;
@@ -869,9 +879,12 @@ class Q_GUI_EXPORT QShortcutEvent : public QEvent
 {
     Q_DECL_EVENT_COMMON(QShortcutEvent)
 public:
+    // Note this is publicly deprecated, but should remain as internal constructor:
     QShortcutEvent(const QKeySequence &key, int id, bool ambiguous = false);
+    QShortcutEvent(const QKeySequence &key, const QShortcut *shortcut = nullptr, bool ambiguous = false);
 
     inline const QKeySequence &key() const { return m_sequence; }
+    // Note this is publicly deprecated, but should remain as internal getter:
     inline int shortcutId() const { return m_shortcutId; }
     inline bool isAmbiguous() const { return m_ambiguous; }
 protected:
@@ -1007,6 +1020,17 @@ private:
     Qt::ApplicationState m_applicationState;
 };
 
+class Q_GUI_EXPORT QChildWindowEvent : public QEvent
+{
+    Q_DECL_EVENT_COMMON(QChildWindowEvent)
+public:
+    explicit QChildWindowEvent(Type type, QWindow *childWindow);
+    QWindow *child() const { return c; }
+
+private:
+    QWindow *c;
+};
+
 QT_END_NAMESPACE
 
 #endif // QEVENT_H
