@@ -1,11 +1,10 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QtGui>
 #include <QtWidgets>
 #include <QtDebug>
-#include <QPair>
 #include <QList>
 #include <QPointer>
 #include <QSignalSpy>
@@ -110,6 +109,8 @@ private slots:
     void dynamicSortOrder();
     void disabledItems();
 
+    void hideWidget();
+
     // task-specific tests below me
     void task178797_activatedOnReturn();
     void task189564_omitNonSelectableItems();
@@ -123,6 +124,7 @@ private slots:
     void QTBUG_52028_tabAutoCompletes();
     void QTBUG_51889_activatedSentTwice();
     void showPopupInGraphicsView();
+    void inheritedEventFilter();
 
 private:
     void filter(bool assync = false);
@@ -1185,6 +1187,32 @@ void tst_QCompleter::disabledItems()
     QVERIFY(!view->isVisible());
 }
 
+void tst_QCompleter::hideWidget()
+{
+    // hiding the widget should hide/close the popup
+    QWidget w;
+    w.setWindowTitle(QLatin1String(QTest::currentTestFunction()));
+    w.setLayout(new QVBoxLayout);
+
+    QLineEdit edit;
+    edit.setCompleter(new QCompleter({ "foo", "bar" }));
+
+    w.layout()->addWidget(&edit);
+
+    const auto pos = w.screen()->availableGeometry().topLeft() + QPoint(200, 200);
+    w.move(pos);
+    w.show();
+    QApplicationPrivate::setActiveWindow(&w);
+    QVERIFY(QTest::qWaitForWindowActive(&w));
+
+    // activate the completer
+    QTest::keyClick(&edit, Qt::Key_F);
+    QVERIFY(edit.completer()->popup());
+    QTRY_VERIFY(edit.completer()->popup()->isVisible());
+    edit.hide();
+    QVERIFY(!edit.completer()->popup()->isVisible());
+}
+
 void tst_QCompleter::task178797_activatedOnReturn()
 {
     if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
@@ -1686,6 +1714,7 @@ void tst_QCompleter::QTBUG_14292_filesystem()
     QTRY_VERIFY(comp.popup()->isVisible());
     QCOMPARE(comp.popup()->model()->rowCount(), 2);
     QApplication::processEvents();
+    QCOMPARE(qApp->focusObject(), &edit); // for QTBUG_108522
     QTest::keyClick(&edit, 'h');
     QCOMPARE(comp.popup()->model()->rowCount(), 2);
     QTest::keyClick(&edit, 'e');
@@ -1863,5 +1892,42 @@ void tst_QCompleter::showPopupInGraphicsView()
     QVERIFY(lineEdit.completer()->popup()->geometry().bottom() < lineEdit.mapToGlobal(QPoint(0, 0)).y());
 }
 
+void tst_QCompleter::inheritedEventFilter()
+{
+    class Completer : public QCompleter
+    {
+    public:
+        explicit Completer(QWidget *parent) : QCompleter(parent)
+        {
+            Q_ASSERT(parent);
+            setPopup(new QListView());
+            popup()->installEventFilter(this);
+        }
+
+        bool m_popupChildAdded = false;
+
+    protected:
+        bool eventFilter(QObject *watched, QEvent *event) override
+        {
+            if (watched == popup() && event->type() == QEvent::ChildAdded)
+                m_popupChildAdded = true;
+
+            return QCompleter::eventFilter(watched, event);
+        }
+    };
+
+    QComboBox comboBox;
+    comboBox.setEditable(true);
+    Completer *completer = new Completer(&comboBox);
+    comboBox.setCompleter(completer);
+
+    // comboBox.show() must not crash with an infinite loop in the event filter
+    comboBox.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&comboBox));
+
+    // Since event orders are platform dependent, only the the ChildAdded event is checked.
+    QVERIFY(QTest::qWaitFor([completer](){return completer->m_popupChildAdded; }));
+}
+
 QTEST_MAIN(tst_QCompleter)
 #include "tst_qcompleter.moc"
