@@ -20,7 +20,7 @@ from pathlib import Path
 from typing import Optional
 
 from qlocalexml import QLocaleXmlReader
-from localetools import unicode2hex, wrap_list, Error, Transcriber, SourceFileEditor, qtbase_root
+from localetools import *
 from iso639_3 import LanguageCodeData
 
 class LocaleKeySorter:
@@ -107,7 +107,7 @@ class StringData:
             raise ValueError(f'Data is too big ({len(self.data)}) for quint16 index to its end!',
                              self.name)
         fd.write(f"\nstatic constexpr char16_t {self.name}[] = {{\n")
-        fd.write(wrap_list(self.data))
+        fd.write(wrap_list(self.data, 12)) # 12 == 100 // len('0xhhhh, ')
         fd.write("\n};\n")
 
 def currencyIsoCodeData(s):
@@ -337,7 +337,11 @@ class LocaleDataWriter (LocaleSourceEditor):
         for key, value in book.items():
             if key == 0:
                 continue
-            out(f'"{value[0]}\\0"\n')
+            enum, name = value[0], value[-1]
+            if names_clash(name, enum):
+                out(f'"{name}\\0" // {enum}\n')
+            else:
+                out(f'"{name}\\0"\n') # Automagically utf-8 encoded
         out(';\n\n')
 
         out(f'static constexpr quint16 {form}_name_index[] = {{\n')
@@ -346,9 +350,8 @@ class LocaleDataWriter (LocaleSourceEditor):
         for key, value in book.items():
             if key == 0:
                 continue
-            name = value[0]
-            out(f'{index:6d}, // {name}\n')
-            index += len(name) + 1
+            out(f'{index:6d}, // {value[0]}\n')
+            index += len(value[-1].encode('utf-8')) + 1
         out('};\n\n')
 
     @staticmethod
@@ -456,9 +459,9 @@ class CalendarDataWriter (LocaleSourceEditor):
         months_data.write(self.writer)
 
 class LocaleHeaderWriter (SourceFileEditor):
-    def __init__(self, path, temp, dupes):
+    def __init__(self, path, temp, enumify):
         super().__init__(path, temp)
-        self.__dupes = dupes
+        self.__enumify = enumify
 
     def languages(self, languages):
         self.__enum('Language', languages, self.__language)
@@ -483,20 +486,10 @@ class LocaleHeaderWriter (SourceFileEditor):
         if suffix is None:
             suffix = name
 
-        out, dupes = self.writer.write, self.__dupes
+        out, enumify = self.writer.write, self.__enumify
         out(f'    enum {name} : ushort {{\n')
         for key, value in book.items():
-            member = value[0].replace('-', ' ')
-            if name == 'Script':
-                # Don't .capitalize() as some names are already camel-case (see enumdata.py):
-                member = ''.join(word[0].upper() + word[1:] for word in member.split())
-                if not member.endswith('Script'):
-                    member += 'Script'
-                if member in dupes:
-                    raise Error(f'The script name "{member}" is messy')
-            else:
-                member = ''.join(member.split())
-                member = member + suffix if member in dupes else member
+            member = enumify(value[0], suffix)
             out(f'        {member} = {key},\n')
 
         out('\n        '
@@ -581,7 +574,7 @@ def main(out, err):
     # qlocale.h
     try:
         with LocaleHeaderWriter(qtsrcdir.joinpath('src/corelib/text/qlocale.h'),
-                                qtsrcdir, reader.dupes) as writer:
+                                qtsrcdir, reader.enumify) as writer:
             writer.languages(reader.languages)
             writer.scripts(reader.scripts)
             writer.territories(reader.territories)
